@warnings
@script custom
@version 2.7

/*
======================================================================
SR Custom Bones is a custom object plugin for displaying bones as
boxes or normal bones with some mroe options


www.Steelronin.com
Scott@steelronin.com  
======================================================================
*/

/*
updates comments
 v0.1.2
  - added better scaling
 v0.1.1:
  - The panel is non module
  - The alpha (opacity) controls have been added, but are currently not in use because the color gets set to black when the alpha is applied
*/ 

@name "SR_Custom_Bones"
scriptName = "SR_Child_Bones_Plus v0.1.2";
copyright = "Copyright (c) 2004 Scott lange";


active;
selBase;
baseVec = <0,0,0>;
scale = 1;
boneParent;
restL = 1;
cursel;

cDisplay = 2;
cScale = 10;
//cColor = <0,0,0>;
//cAlpha = 1.0;
cFilled = false;
cScaleX = 1;
cScaleY = 1;
cScaleZ = 1;



create: ma
{
    selBase = ma;
    
    //cScale = selbase.getScale(0);
    cScaleX = selBase.getScaling(0).x;
    cScaleY = selBase.getScaling(0).y;
    cScaleZ = selBase.getScaling(0).z;
    
    if(ma.parent)
    {
        boneParent = ma.parent;
        if(boneParent.genus == 4)
        {
            restL = boneParent.restlength;
            active = boneParent.flags[1];
        }
    }
    else
        error("No parent bone present");
    setdesc("SR_Custom_Bones_v0.1.2");
}


process: ca
{
    if(active == nil)
    {
        selBase = Mesh(0);
        boneParent = selBase.parent;
        active = boneParent.flags[1];
    }
    else
        active = boneParent.flags[1];
        
    scale = (restL * .075);
    
    Scale(cScaleX,cScaleY,cScaleZ);

    //ca.setColor(cColor.x,cColor.y,cColor.z,0.1); //Add this later when color works
	
    // box points
    boxVert = @
    <baseVec.x - scale,baseVec.y + scale,baseVec.z>,              // 1 top upper left
    <baseVec.x + scale,baseVec.y + scale,baseVec.z>,              // 2 top upper right
    <baseVec.x + scale,baseVec.y - scale,baseVec.z>,               // 3 top lower right
    <baseVec.x - scale,baseVec.y - scale,baseVec.z>,              // 4 top lower left
    <baseVec.x - scale,baseVec.y + scale,baseVec.z + restL>,      // 5 bottom upper left
    <baseVec.x + scale,baseVec.y + scale,baseVec.z + restL>,      // 6 bottom upper right
    <baseVec.x + scale,baseVec.y - scale,baseVec.z + restL>,      // 7 bottom lower right
    <baseVec.x - scale,baseVec.y - scale,baseVec.z + restL> @;      // 8 bottom lower left

    //box edges
    boxEdge = @
        1,2,
        2,3,
        3,4,
        4,1,
        5,6,
        6,7,
        7,8,
        8,5,
        1,5,
        2,6,
        3,7,
        4,8 @;

    //box tries
    boxTries = @
        1,5,2,
        2,5,6,
        2,6,7,
        2,7,3,
        3,7,8,
        3,8,4,
        4,8,5,
        4,5,1,
        1,2,3,
        4,4,1,
        5,6,7,
        7,8,5 @;
    
    // bone vectors    
    boneVert = @
    <baseVec.x,baseVec.y,baseVec.z>,                               // 1 top 
    <baseVec.x,baseVec.y + scale,baseVec.z + scale>,               // 2 top upper
    <baseVec.x + scale,baseVec.y,baseVec.z + scale>,               // 3 top right
    <baseVec.x,baseVec.y - scale,baseVec.z + scale>,               // 4 top lower
    <baseVec.x - scale,baseVec.y,baseVec.z + scale>,               // 5 top left
    <baseVec.x,baseVec.y,baseVec.z + restL> @;                     // 6 bottom 

    boneEdge = @
        1,2,
        1,3,
        1,4,
        1,5,
        2,3,
        3,4,
        4,5,
        5,2,
        2,6,
        3,6,
        4,6,
        5,6 @;

    boneTries = @
        1,2,3,
        1,3,4,
        1,4,5,
        1,5,2,
        2,6,3,
        3,6,4,
        4,6,5,
        5,6,2 @;    
    
        
        

    if(active == true) //active bone will display custom bone
    {
        // bone wire
        if(cDisplay == 1 && cFilled == 0)
        {
            for(x = 1; x < 24; x += 2)
                ca.drawLine(boneVert[boneEdge[x]],boneVert[boneEdge[x+1]]);

        }
        
        // bone shaded
        if(cDisplay == 1 && cFilled == 1)
        {
            for(x = 1; x < 23; x += 2)
            {
                ca.drawTriangle(boneVert[boneTries[x]],boneVert[boneTries[x+1]],boneVert[boneTries[x+2]]);
                --x;
            }
        }
        
        // box wire
        if(cDisplay == 2 && cFilled == 0)
        {
            for(x = 1; x < 24; x += 2)
                ca.drawLine(boxVert[boxEdge[x]],boxVert[boxEdge[x+1]]);

        }
        
        // box shaded
        if(cDisplay == 2 && cFilled == 1)
        {
            for(x = 1; x < 35; x += 2)
            {
                ca.drawTriangle(boxVert[boxTries[x]],boxVert[boxTries[x+1]],boxVert[boxTries[x+2]]);
                --x;
            }
        }        
    }



}

load: what,io
{
    if(what == SCENEMODE) // processing an ASCII scene file
    {
        //active = integer(io.read());
        baseVec = vector(io.read());
        restL = number(io.read());
        cDisplay = integer(io.read());
        scale = integer(io.read());
        cScaleX = integer(io.read());
        cScaleY = integer(io.read());
        cScaleZ = integer(io.read());
        //cColor = vector(io.read());
        cFilled = integer(io.read());
        
        setdesc("SR_Custom_Bones_v0.1.2");
    }
}

save: what,io
{
    if(what == SCENEMODE)
    {
        //io.writeln(active);
        io.writeln(baseVec);
        io.writeln(restL);
        io.writeln(cDisplay);
        io.writeln(scale);
        io.writeln(cScaleX);
        io.writeln(cScaleY);
        io.writeln(cScaleZ);
        //io.writeln(cColor);
        io.writeln(cFilled);
    }
}

options
{
    
    reqbegin("SR Custom Bone");

    c1 = ctlbutton("Update RestLength",120,"update_rest");

    c2 = ctlchoice("Display",cDisplay,@"Bone","Box"@);
	
	ctlrefresh(c2,"refc2");

    //c4 = ctlcolor("Color",cColor);

	//ctlrefresh(c4,"refc4");
	
	//c6 = ctlcolor("Alpha",cAlpha);

	//ctlrefresh(c6,"refc6");

    c5 = ctlcheckbox("Filled",cFilled);
        ctlrefresh(c5,"refc5");
    
    c6 = ctlslider("ScaleX",cScaleX,1,200);
        ctlrefresh(c6,"refc6");
        
    c7 = ctlslider("ScaleY",cScaleY,1,200);
        ctlrefresh(c7,"refc7");
        
    c8 = ctlslider("ScaleZ",cScaleZ,1,200);
        ctlrefresh(c8,"refc8");
	


	reqopen();
}

refc2: value
{
	cDisplay = value;
}

refc6: value
{
	cScaleX = value * .1;
}
refc7: value
{
	cScaleY = value * .1;
}
refc8: value
{
	cScaleZ = value * .1;
}

/*refc4: value
{
	cColor = value;
}*/

/*refc6: value
{
	cAlpha = value;
}*/

refc5: value
{
	cFilled = value;
}

update_rest
{
    if(selBase == nil)
    {
        selBase = Mesh(0);
        boneParent = selBase.parent;
        restL = boneParent.restlength;
        active = boneParent.flags[1];
    }
    else
    {
        restL = boneParent.restlength;
        active = boneParent.flags[1];
    }
}